//////////////////////////////////////////////////////////////////////////////////////////
// IFSWITCHER
//////////////////////////////////////////////////////////////////////////////////////////
// 08/21/2003 - 23:54:51
// (C) 2003 Romain GUY <romain.guy@jext.org>
// Code(r) n5
//
// Gestionnaire de configurations des cartes rseaux pour Windows
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
// IMPORT DES NAMESPACES NECESSAIRES
//////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Data;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.IO;
using System.Management;
using System.Windows.Forms;
using System.Xml;

//////////////////////////////////////////////////////////////////////////////////////////
// DECLARATION DE NOTRE NAMESPACE
//////////////////////////////////////////////////////////////////////////////////////////

namespace PossePress.Coder.IfSwitcher
{
	/// <summary>
	/// IfSwitcherForm reprsente la fentre principale du logiciel ainsi que son moteur.
	/// La fentre contient la liste des interfaces rseau et un panneau permettant de
	/// modifier chaque interface de manire indpendante.
	/// </summary>
	public class IfSwitcherForm : System.Windows.Forms.Form
	{
		/// <summary>
		/// Objet de gestion des interfaces rseau.
		/// </summary>
		private ManagementClass networkIfClass;
		private ManagementObjectCollection networkIfs;

		private DataSet	ifDataSet;

		private const int MODE 			= 1;
		private const int IP_ADDRESS 	= 2;
		private const int GATE_WAY 		= 3;
		private const int NET_MASK 		= 4;
		private const int DNS_1 		= 5;
		private const int DNS_2 		= 6;

		/// <summary>
		/// Composants du GUI gnr par le Designer.
		/// </summary>
		private System.Windows.Forms.GroupBox grpNetworkIfs;
		private System.Windows.Forms.DataGrid dataGrid;
		private System.Windows.Forms.GroupBox grpIfSettings;
		private System.Windows.Forms.Label lblMode;
		private System.Windows.Forms.ComboBox cmbMode;
		private System.Windows.Forms.Label lblDNS2;
		private System.Windows.Forms.TextBox txtDNS2;
		private System.Windows.Forms.Label lblDNS1;
		private System.Windows.Forms.TextBox txtDNS1;
		private System.Windows.Forms.Label lblNetmask;
		private System.Windows.Forms.TextBox txtNetmask;
		private System.Windows.Forms.Label lblIPAddress;
		private System.Windows.Forms.TextBox txtIPAddress;
		private System.Windows.Forms.Button btnQuit;
		private System.Windows.Forms.Button btnSet;
		private System.Windows.Forms.Button btnSave;
		private System.Windows.Forms.Button btnRefresh;
		private System.Windows.Forms.Button btnLoad;
		private System.Windows.Forms.Label lblGateway;
		private System.Windows.Forms.TextBox txtGateway;

		public IfSwitcherForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			cmbMode.SelectedIndex = 0;
			// rcupration de la classe de gestion des interfaces rseau
			this.networkIfClass = new ManagementClass("Win32_NetworkAdapterConfiguration");
			RefreshInterfaces();
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the codeprivate void InitializeComponent()
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(IfSwitcherForm));
			this.grpNetworkIfs = new System.Windows.Forms.GroupBox();
			this.dataGrid = new System.Windows.Forms.DataGrid();
			this.grpIfSettings = new System.Windows.Forms.GroupBox();
			this.lblGateway = new System.Windows.Forms.Label();
			this.txtGateway = new System.Windows.Forms.TextBox();
			this.lblMode = new System.Windows.Forms.Label();
			this.cmbMode = new System.Windows.Forms.ComboBox();
			this.lblDNS2 = new System.Windows.Forms.Label();
			this.txtDNS2 = new System.Windows.Forms.TextBox();
			this.lblDNS1 = new System.Windows.Forms.Label();
			this.txtDNS1 = new System.Windows.Forms.TextBox();
			this.lblNetmask = new System.Windows.Forms.Label();
			this.txtNetmask = new System.Windows.Forms.TextBox();
			this.lblIPAddress = new System.Windows.Forms.Label();
			this.txtIPAddress = new System.Windows.Forms.TextBox();
			this.btnQuit = new System.Windows.Forms.Button();
			this.btnSet = new System.Windows.Forms.Button();
			this.btnSave = new System.Windows.Forms.Button();
			this.btnRefresh = new System.Windows.Forms.Button();
			this.btnLoad = new System.Windows.Forms.Button();
			this.grpNetworkIfs.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.dataGrid)).BeginInit();
			this.grpIfSettings.SuspendLayout();
			this.SuspendLayout();
			// 
			// grpNetworkIfs
			// 
			this.grpNetworkIfs.Controls.Add(this.dataGrid);
			this.grpNetworkIfs.Location = new System.Drawing.Point(0, 0);
			this.grpNetworkIfs.Name = "grpNetworkIfs";
			this.grpNetworkIfs.Size = new System.Drawing.Size(376, 184);
			this.grpNetworkIfs.TabIndex = 0;
			this.grpNetworkIfs.TabStop = false;
			this.grpNetworkIfs.Text = "Network Interfaces";
			// 
			// dataGrid
			// 
			this.dataGrid.AllowSorting = false;
			this.dataGrid.BorderStyle = System.Windows.Forms.BorderStyle.None;
			this.dataGrid.CaptionVisible = false;
			this.dataGrid.DataMember = "";
			this.dataGrid.HeaderForeColor = System.Drawing.SystemColors.ControlText;
			this.dataGrid.Location = new System.Drawing.Point(8, 16);
			this.dataGrid.Name = "dataGrid";
			this.dataGrid.ReadOnly = true;
			this.dataGrid.Size = new System.Drawing.Size(360, 160);
			this.dataGrid.TabIndex = 0;
			this.dataGrid.Click += new System.EventHandler(this.dataGrid_Clicked);
			// 
			// grpIfSettings
			// 
			this.grpIfSettings.Controls.Add(this.lblGateway);
			this.grpIfSettings.Controls.Add(this.txtGateway);
			this.grpIfSettings.Controls.Add(this.lblMode);
			this.grpIfSettings.Controls.Add(this.cmbMode);
			this.grpIfSettings.Controls.Add(this.lblDNS2);
			this.grpIfSettings.Controls.Add(this.txtDNS2);
			this.grpIfSettings.Controls.Add(this.lblDNS1);
			this.grpIfSettings.Controls.Add(this.txtDNS1);
			this.grpIfSettings.Controls.Add(this.lblNetmask);
			this.grpIfSettings.Controls.Add(this.txtNetmask);
			this.grpIfSettings.Controls.Add(this.lblIPAddress);
			this.grpIfSettings.Controls.Add(this.txtIPAddress);
			this.grpIfSettings.Location = new System.Drawing.Point(384, 0);
			this.grpIfSettings.Name = "grpIfSettings";
			this.grpIfSettings.Size = new System.Drawing.Size(192, 184);
			this.grpIfSettings.TabIndex = 1;
			this.grpIfSettings.TabStop = false;
			this.grpIfSettings.Text = "Interface Settings";
			// 
			// lblGateway
			// 
			this.lblGateway.Location = new System.Drawing.Point(8, 72);
			this.lblGateway.Name = "lblGateway";
			this.lblGateway.Size = new System.Drawing.Size(72, 16);
			this.lblGateway.TabIndex = 11;
			this.lblGateway.Text = "Gateway:";
			// 
			// txtGateway
			// 
			this.txtGateway.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtGateway.Location = new System.Drawing.Point(96, 72);
			this.txtGateway.MaxLength = 15;
			this.txtGateway.Name = "txtGateway";
			this.txtGateway.Size = new System.Drawing.Size(88, 20);
			this.txtGateway.TabIndex = 10;
			this.txtGateway.Text = "255.255.255.255";
			// 
			// lblMode
			// 
			this.lblMode.Location = new System.Drawing.Point(8, 152);
			this.lblMode.Name = "lblMode";
			this.lblMode.Size = new System.Drawing.Size(72, 16);
			this.lblMode.TabIndex = 9;
			this.lblMode.Text = "IP Mode:";
			// 
			// cmbMode
			// 
			this.cmbMode.DisplayMember = "1";
			this.cmbMode.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbMode.Items.AddRange(new object[] {
						"DHCP",
						"Fixed IP"});
			this.cmbMode.Location = new System.Drawing.Point(96, 152);
			this.cmbMode.Name = "cmbMode";
			this.cmbMode.Size = new System.Drawing.Size(88, 21);
			this.cmbMode.Sorted = true;
			this.cmbMode.TabIndex = 8;
			this.cmbMode.ValueMember = "1";
			// 
			// lblDNS2
			// 
			this.lblDNS2.Location = new System.Drawing.Point(8, 120);
			this.lblDNS2.Name = "lblDNS2";
			this.lblDNS2.Size = new System.Drawing.Size(72, 16);
			this.lblDNS2.TabIndex = 7;
			this.lblDNS2.Text = "DNS 2:";
			// 
			// txtDNS2
			// 
			this.txtDNS2.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtDNS2.Location = new System.Drawing.Point(96, 120);
			this.txtDNS2.MaxLength = 15;
			this.txtDNS2.Name = "txtDNS2";
			this.txtDNS2.Size = new System.Drawing.Size(88, 20);
			this.txtDNS2.TabIndex = 6;
			this.txtDNS2.Text = "255.255.255.255";
			// 
			// lblDNS1
			// 
			this.lblDNS1.Location = new System.Drawing.Point(8, 96);
			this.lblDNS1.Name = "lblDNS1";
			this.lblDNS1.Size = new System.Drawing.Size(72, 16);
			this.lblDNS1.TabIndex = 5;
			this.lblDNS1.Text = "DNS 1:";
			// 
			// txtDNS1
			// 
			this.txtDNS1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtDNS1.Location = new System.Drawing.Point(96, 96);
			this.txtDNS1.MaxLength = 15;
			this.txtDNS1.Name = "txtDNS1";
			this.txtDNS1.Size = new System.Drawing.Size(88, 20);
			this.txtDNS1.TabIndex = 4;
			this.txtDNS1.Text = "255.255.255.255";
			// 
			// lblNetmask
			// 
			this.lblNetmask.Location = new System.Drawing.Point(8, 48);
			this.lblNetmask.Name = "lblNetmask";
			this.lblNetmask.Size = new System.Drawing.Size(72, 16);
			this.lblNetmask.TabIndex = 3;
			this.lblNetmask.Text = "Netmask:";
			// 
			// txtNetmask
			// 
			this.txtNetmask.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtNetmask.Location = new System.Drawing.Point(96, 48);
			this.txtNetmask.MaxLength = 15;
			this.txtNetmask.Name = "txtNetmask";
			this.txtNetmask.Size = new System.Drawing.Size(88, 20);
			this.txtNetmask.TabIndex = 2;
			this.txtNetmask.Text = "255.255.255.255";
			// 
			// lblIPAddress
			// 
			this.lblIPAddress.Location = new System.Drawing.Point(8, 24);
			this.lblIPAddress.Name = "lblIPAddress";
			this.lblIPAddress.Size = new System.Drawing.Size(72, 16);
			this.lblIPAddress.TabIndex = 1;
			this.lblIPAddress.Text = "IP Address:";
			// 
			// txtIPAddress
			// 
			this.txtIPAddress.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtIPAddress.Location = new System.Drawing.Point(96, 24);
			this.txtIPAddress.MaxLength = 15;
			this.txtIPAddress.Name = "txtIPAddress";
			this.txtIPAddress.Size = new System.Drawing.Size(88, 20);
			this.txtIPAddress.TabIndex = 0;
			this.txtIPAddress.Text = "255.255.255.255";
			// 
			// btnQuit
			// 
			this.btnQuit.BackColor = System.Drawing.Color.Lavender;
			this.btnQuit.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnQuit.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.btnQuit.Location = new System.Drawing.Point(488, 192);
			this.btnQuit.Name = "btnQuit";
			this.btnQuit.Size = new System.Drawing.Size(56, 24);
			this.btnQuit.TabIndex = 2;
			this.btnQuit.Text = "&Quit";
			this.btnQuit.Click += new System.EventHandler(this.btnQuit_Click);
			// 
			// btnSet
			// 
			this.btnSet.BackColor = System.Drawing.Color.Azure;
			this.btnSet.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnSet.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.btnSet.Location = new System.Drawing.Point(408, 192);
			this.btnSet.Name = "btnSet";
			this.btnSet.Size = new System.Drawing.Size(56, 24);
			this.btnSet.TabIndex = 3;
			this.btnSet.Text = "&Set";
			this.btnSet.Click += new System.EventHandler(this.btnSet_Click);
			// 
			// btnSave
			// 
			this.btnSave.BackColor = System.Drawing.Color.AntiqueWhite;
			this.btnSave.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnSave.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.btnSave.Location = new System.Drawing.Point(224, 192);
			this.btnSave.Name = "btnSave";
			this.btnSave.Size = new System.Drawing.Size(56, 24);
			this.btnSave.TabIndex = 4;
			this.btnSave.Text = "&Save";
			this.btnSave.Click += new System.EventHandler(this.btnSave_Click);
			// 
			// btnRefresh
			// 
			this.btnRefresh.BackColor = System.Drawing.Color.PeachPuff;
			this.btnRefresh.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnRefresh.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.btnRefresh.Location = new System.Drawing.Point(96, 192);
			this.btnRefresh.Name = "btnRefresh";
			this.btnRefresh.Size = new System.Drawing.Size(56, 24);
			this.btnRefresh.TabIndex = 5;
			this.btnRefresh.Text = "&Refresh";
			this.btnRefresh.Click += new System.EventHandler(this.btnRefresh_Click);
			// 
			// btnLoad
			// 
			this.btnLoad.BackColor = System.Drawing.Color.Bisque;
			this.btnLoad.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnLoad.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.btnLoad.Location = new System.Drawing.Point(160, 192);
			this.btnLoad.Name = "btnLoad";
			this.btnLoad.Size = new System.Drawing.Size(56, 24);
			this.btnLoad.TabIndex = 6;
			this.btnLoad.Text = "&Load";
			this.btnLoad.Click += new System.EventHandler(this.btnLoad_Click);
			// 
			// IfSwitcherForm
			// 
			this.AcceptButton = this.btnSet;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.btnQuit;
			this.ClientSize = new System.Drawing.Size(576, 221);
			this.Controls.Add(this.btnLoad);
			this.Controls.Add(this.btnRefresh);
			this.Controls.Add(this.btnSave);
			this.Controls.Add(this.btnSet);
			this.Controls.Add(this.btnQuit);
			this.Controls.Add(this.grpIfSettings);
			this.Controls.Add(this.grpNetworkIfs);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.Name = "IfSwitcherForm";
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "IfSwitcher - Code(r) n5";
			this.grpNetworkIfs.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.dataGrid)).EndInit();
			this.grpIfSettings.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion

		//////////////////////////////////////////////////////////////////////////////////////
		// GESTIONNAIRES D'EVENEMENTS
		//////////////////////////////////////////////////////////////////////////////////////

		private void btnQuit_Click(object sender, System.EventArgs e)
		{
			this.Close();
			System.Environment.Exit(0);
		}

		private void btnSet_Click(object sender, System.EventArgs e)
		{
			SetInterface(dataGrid.CurrentRowIndex);
		}
		
		private void btnSave_Click(object sender, System.EventArgs e)
		{
            SaveFileDialog fileDialog = new SaveFileDialog();

            fileDialog.Filter = "Fichiers XML (*.xml)|*.xml";
            fileDialog.RestoreDirectory = true;

            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                Stream fileStream;
                if ((fileStream = fileDialog.OpenFile()) != null)
                    SaveInterfacesAsXML(fileStream);
            }
		}

		private void btnLoad_Click(object sender, System.EventArgs e)
		{
            this.Cursor = Cursors.WaitCursor;

            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.Filter = "Fichiers XML (*.xml)|*.xml";
            fileDialog.RestoreDirectory = true;

            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                Stream fileStream;
                if ((fileStream = fileDialog.OpenFile()) != null)
                    LoadInterfacesFromXML(fileStream);
            }

            this.InvokeOnClick(this.btnRefresh, new EventArgs());
            this.Cursor = Cursors.Arrow;
		}
		
		private void btnRefresh_Click(object sender, System.EventArgs e)
		{
			RefreshInterfaces();
		}

		private void dataGrid_Clicked(object sender, System.EventArgs e)
		{
			SelectInterface(dataGrid.CurrentRowIndex);
		}

		//////////////////////////////////////////////////////////////////////////////////////
		// METHODES DE GESTION DE LA CONFIGURATION DES INTERFACES RESEAU
		//////////////////////////////////////////////////////////////////////////////////////

        /// <summary>
        /// Modifie les proprits des interfaces rseau en fonction d'un fichier XML.
        /// </summary>

        private void LoadInterfacesFromXML(Stream fileStream)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(fileStream);

            foreach (XmlElement e in doc.GetElementsByTagName("interface"))
            {
                string ifName = e.GetElementsByTagName("description")[0].FirstChild.Value;
                ManagementObject networkIf = null;
                this.networkIfs = networkIfClass.GetInstances();
    
                foreach (ManagementObject nif in networkIfs)
                {
                    if (nif["description"].ToString() == ifName)
                    {
                        networkIf = nif;
                        break;
                    }
                }

                if (networkIf != null)
                {
                    string dhcp    = e.GetElementsByTagName("dhcp")[0].FirstChild.Value;
                    string ip      = e.GetElementsByTagName("ip")[0].FirstChild.Value;
                    string netmask = e.GetElementsByTagName("netmask")[0].FirstChild.Value;
                    string gateway = e.GetElementsByTagName("gateway")[0].FirstChild.Value;
                    string dns1    = e.GetElementsByTagName("dns1")[0].FirstChild.Value;
                    string dns2    = e.GetElementsByTagName("dns2")[0].FirstChild.Value;

                    ChangeInterfaceProperties(networkIf,
                                              ip, netmask,
                                              gateway, dns1, dns2,
                                              (dhcp == "True" ? 0 : 1));
                }
            }

            fileStream.Close();
        }

        /// <summary>
        /// Enregistre la liste des interfaces dans un document XML.
        /// </summary>

        private void SaveInterfacesAsXML(Stream fileStream)
        {
            XmlDocument doc = new XmlDocument();
            XmlElement root = doc.CreateElement("ifswitcher");
            doc.AppendChild(root);

            XmlElement subElement, e;
            this.networkIfs = networkIfClass.GetInstances();

            foreach (ManagementObject networkIf in this.networkIfs)
           	{
                subElement = doc.CreateElement("interface");

                e = doc.CreateElement("description");
                e.AppendChild(doc.CreateTextNode(networkIf["description"].ToString()));
                subElement.AppendChild(e);

                e = doc.CreateElement("dhcp");
				e.AppendChild(doc.CreateTextNode(networkIf["DHCPEnabled"].ToString()));
                subElement.AppendChild(e);

                e = doc.CreateElement("ip");
				e.AppendChild(doc.CreateTextNode(GetIP(networkIf["IPAddress"])));
                subElement.AppendChild(e);

                e = doc.CreateElement("netmask");
				e.AppendChild(doc.CreateTextNode(GetIP(networkIf["IPSubnet"])));
                subElement.AppendChild(e);

                e = doc.CreateElement("gateway");
				e.AppendChild(doc.CreateTextNode(GetIP(networkIf["DefaultIPGateway"])));
                subElement.AppendChild(e);

                e = doc.CreateElement("dns1");
				e.AppendChild(doc.CreateTextNode(GetDNSPrimary(networkIf["DNSServerSearchOrder"])));
                subElement.AppendChild(e);

                e = doc.CreateElement("dns2");
				e.AppendChild(doc.CreateTextNode(GetDNSSecondary(networkIf["DNSServerSearchOrder"])));
                subElement.AppendChild(e);

                root.AppendChild(subElement);
           	}

            doc.Save(fileStream);
            fileStream.Close();
        }

        /// <summary>
        /// Active les nouveaux paramtres pour l'interface rseau situe  la position
        /// "index" de la liste des instances de la classe "networkIfClass".
        /// </summary>

		private void SetInterface(int index)
		{
			int counter = 0;
			ManagementObject networkIf = null;
            this.networkIfs = networkIfClass.GetInstances();

			foreach (ManagementObject nif in this.networkIfs)
			{
				if (counter++ == index)
				{
					networkIf = nif;
					break;
				}
			}

			DataTable t = ifDataSet.Tables["Interfaces"];
			t.Rows[index][IP_ADDRESS] = txtIPAddress.Text;
			t.Rows[index][NET_MASK]   = txtNetmask.Text;
			t.Rows[index][GATE_WAY]   = txtGateway.Text;
			t.Rows[index][DNS_1]      = txtDNS1.Text;
			t.Rows[index][DNS_2]      = txtDNS2.Text;
			t.Rows[index][MODE]       = (cmbMode.SelectedIndex == 0 ? "DHCP" : "Static");

            this.Cursor = Cursors.WaitCursor;
            ChangeInterfaceProperties(networkIf,
                                      txtIPAddress.Text, txtNetmask.Text,
                                      txtGateway.Text, txtDNS1.Text, txtDNS2.Text,
                                      cmbMode.SelectedIndex);
            this.RefreshInterfaces();
            this.Cursor = Cursors.Arrow;
		}

        /// <summary>
        /// Change les proprits de l'interface rseau passe en paramtre. Le paramtre
        /// "mode" vaut 0 pour DHCP ou 1 pour IP fixe.
        /// </summary>

        private void ChangeInterfaceProperties(ManagementObject networkIf,
                                               string ip, string netmask,
                                               string gateway, string dns1, string dns2,
                                               int mode)
        {
            ManagementBaseObject inParams;
            ManagementBaseObject outParams;

            // on passe l'interface en DHCP
            if (mode == 0)
            {
                outParams = networkIf.InvokeMethod("EnableDHCP", null, null);
                if (outParams["returnValue"].ToString() == "0")
                    outParams = networkIf.InvokeMethod("RenewDHCPLease", null, null);

            } else {
                inParams = this.networkIfClass.GetMethodParameters("SetGateways");
                inParams["DefaultIPGateway"]  = new object[] { gateway };
                inParams["GatewayCostMetric"] = new object[] { 1 };
                outParams = networkIf.InvokeMethod("SetGateways", inParams, null);

                inParams = this.networkIfClass.GetMethodParameters("EnableStatic");
                inParams["IPAddress"]  = new object[] { ip };
                inParams["SubnetMask"] = new object[] { netmask };
                outParams = networkIf.InvokeMethod("EnableStatic", inParams, null);

                inParams = this.networkIfClass.GetMethodParameters("SetDNSServerSearchOrder");
                inParams["DNSServerSearchOrder"]  = new object[] { dns1, dns2 };
                outParams = networkIf.InvokeMethod("SetDNSServerSearchOrder", inParams, null);
            }
        }

		/// <summary>
		/// Met  jour l'affichage du panneau de configuration des interfaces en fonction
        /// de l'interface slectionne.
		/// </summary>

		private void SelectInterface(int index)
		{
			DataTable t = ifDataSet.Tables["Interfaces"];

			txtIPAddress.Text		= (t.Rows[index][IP_ADDRESS]).ToString();
			txtNetmask.Text			= (t.Rows[index][NET_MASK]).ToString();
			txtGateway.Text			= (t.Rows[index][GATE_WAY]).ToString();
			txtDNS1.Text 		 	= (t.Rows[index][DNS_1]).ToString();
			txtDNS2.Text 			= (t.Rows[index][DNS_2]).ToString();
			cmbMode.SelectedIndex	= (t.Rows[index][MODE]).ToString() == "DHCP" ? 0 : 1;
		}

		/// <summary>
		/// Mise  jour de la liste des interfaces.
		/// </summary>

		private void RefreshInterfaces()
		{
			FeedDataGrid();
			SelectInterface(0);
		}

		/// <summary>
		/// Remplissage de la table des interfaces.
		/// </summary>

		private void FeedDataGrid()
        {
			this.networkIfs = networkIfClass.GetInstances();

			ifDataSet = new DataSet("ifDataSet");

           	DataTable tIfNames = new DataTable("Interfaces");

			DataColumn cIfName	  = new DataColumn("Name");
			DataColumn cIfMode	  = new DataColumn("Mode");
			DataColumn cIfIP	  = new DataColumn("IP");
           	DataColumn cIfMask	  = new DataColumn("Mask");
           	DataColumn cIfGateway = new DataColumn("Gateway");
			DataColumn cIfDNS1	  = new DataColumn("DNS1");
			DataColumn cIfDNS2	  = new DataColumn("DNS2");
    
           	tIfNames.Columns.Add(cIfName);
           	tIfNames.Columns.Add(cIfMode);
           	tIfNames.Columns.Add(cIfIP);
           	tIfNames.Columns.Add(cIfGateway);
           	tIfNames.Columns.Add(cIfMask);
           	tIfNames.Columns.Add(cIfDNS1);
           	tIfNames.Columns.Add(cIfDNS2);
    
           	ifDataSet.Tables.Add(tIfNames);
    
           	DataRow rowIf;
    
			foreach (ManagementObject networkIf in this.networkIfs)
           	{
				rowIf	 	 	 = tIfNames.NewRow();
				rowIf["Name"]	 = networkIf["description"];
				rowIf["Mode"]	 = ((bool) networkIf["DHCPEnabled"]) ? "DHCP" : "Static";
				rowIf["IP"]	 	 = GetIP(networkIf["IPAddress"]);
				rowIf["Mask"]	 = GetIP(networkIf["IPSubnet"]);
				rowIf["Gateway"] = GetIP(networkIf["DefaultIPGateway"]);
				rowIf["DNS1"]	 = GetDNSPrimary(networkIf["DNSServerSearchOrder"]);
				rowIf["DNS2"]	 = GetDNSSecondary(networkIf["DNSServerSearchOrder"]);

				tIfNames.Rows.Add(rowIf);
           	}

			this.dataGrid.SetDataBinding(ifDataSet, "Interfaces");
		}
    
        /// <summary>
        /// Renvoie le DNS primaire pour un objet de type IP.
        /// </summary>
    
        private string GetDNSPrimary(object ipObject)
        {
           	return GetIP(ipObject);
        }
    
        /// <summary>
        /// Renvoie le DNS secondaire pour un objet de type IP.
        /// </summary>
    
        private string GetDNSSecondary(object ipObject)
        {
           	return GetIP(ipObject, 1);
        }
    
        /// <summary>
        /// 
        /// </summary>
    
        private string GetIP(object ipObject)
        {
           	return GetIP(ipObject, 0);
        }
    
        /// <summary>
        /// Renvoie une chane correspondant l'objet IP fourni.
        /// </summary>
    
        private string GetIP(object ipObject, int index)
        {
           	return ipObject == null ? "0.0.0.0" : ((string[]) ipObject)[index];
		}

		/// <summary>
		/// Point d'entre.
		/// </summary>
		[STAThread]
		static void Main()
		{
			Application.Run(new IfSwitcherForm());
		}
	}
}